#!/usr/bin/env python3
# -*-coding:utf8-*-
#机械臂协议解析基类

from abc import ABC, abstractmethod
from enum import Enum, auto
import ctypes
import struct
from typing_extensions import (
    Literal,
)

class C_PiperParserBase(ABC):
    '''
    Piper机械臂数据解析基类
    '''
    '''
    Piper Robot Arm Data Parsing Base Class
    '''
    class ProtocolVersion(Enum):
        '''
        协议版本枚举,需要在派生类中指定
        '''
        '''
        Protocol Version Enumeration, needs to be specified in the derived class.
        '''
        ARM_PROTOCOL_UNKNOWN = auto()
        ARM_PROROCOL_V1 = auto()
        def __str__(self):
            return f"{self.name} (0x{self.value:X})"
        def __repr__(self):
            return f"{self.name}: 0x{self.value:X}"
    
    def __init__(self) -> None:
        super().__init__()
    
    @abstractmethod
    def DecodeMessage(self):
        '''
        解码消息,将can数据帧转为设定的类型
        '''
        '''
        Decode the message, converting the CAN data frame into the specified type.
        '''
        pass
    
    @abstractmethod
    def EncodeMessage(self):
        '''
        将消息转为can数据帧
        
        只将输入数据转换为can数据的id和data, 没有为can message赋值channel、dlc、is_extended_id
        '''
        '''
        Convert the message into a CAN data frame.

        This function only converts the input data into the id and data of the CAN message, without assigning values to channel, dlc, or is_extended_id for the CAN message.
        '''
        pass

    @abstractmethod
    def GetParserProtocolVersion(self):
        '''
        获取当前协议版本
        '''
        '''
        Get the current protocol version.
        '''
        pass
    
    def ConvertToNegative_8bit(self, value: int, signed:bool=True) -> int:
        '''
        将输入的整数转换为8位整数。
        输入 value 范围:[0,255]
        如果 signed 为 True,则转换为8位有符号整数。[-128, 127]
        如果 signed 为 False,则转换为8位无符号整数。[0, 255]
        '''
        '''
        Convert the input integer to an 8-bit integer.
        input value range:[0,255]
        If signed is True, it will be converted to a 32-bit signed integer. [-128, 127]
        If signed is False, it will be converted to a 32-bit unsigned integer. [0, 255]
        '''
        # 范围检查
        if not (0 <= value <= 255):
            print("Error ConvertToNegative_8bit:  Input value exceeds the range [0, 255].")
        # 转换成 8 位无符号整数
        value &= 0xFF  # 将 value 转换成 8 位无符号整数
        if signed:
            if value & 0x80:  # 检查符号位
                value -= 0x100  # 如果符号位为 1，表示负数，需要减去 2^8
        return value

    def ConvertToNegative_int8_t(value: int) -> int:
        '''
        将输入的整数转换为8位有符号整数。
        输入 value 范围:[0,255]
        return范围: [-128, 127]
        '''
        '''
        Convert the input integer to an 8-bit signed integer.
        input value range:[0,255]
        return Range: [-128, 127]
        '''
        # 范围检查
        if not (0 <= value <= 255):
            print("Error ConvertToNegative_int8_t: Input value exceeds the range [0, 255].")
        # 转换成 8 位无符号整数
        value &= 0xFF  # 将 value 转换成 8 位无符号整数
        if value & 0x80:  # 检查符号位
            value -= 0x100  # 如果符号位为 1，表示负数，需要减去 2^8
        return value

    def ConvertToNegative_uint8_t(value: int) -> int:
        '''
        将输入的整数转换为8位无符号整数。
        输入 value 范围:[0,255]
        return 范围: [0, 255]
        '''
        '''
        Convert the input integer to an 8-bit unsigned integer.
        input value range:[0,255]
        return Range: [0, 255]
        '''
        # 范围检查
        if not (0 <= value <= 255):
            print("Error ConvertToNegative_uint8_t: Input value exceeds the range [0, 255].")
        # 转换成 8 位无符号整数
        value &= 0xFF  # 将 value 转换成 8 位无符号整数
        return value

    def ConvertToNegative_16bit(self, value: int, signed:bool=True) -> int:
        '''
        将输入的整数转换为16位整数。
        输入 value 范围:[0,65535]
        如果 signed 为 True,则转换为16位有符号整数。[-32768, 32767]
        如果 signed 为 False,则转换为16位无符号整数。[0, 65535]
        '''
        '''
        Convert the input integer to a 16-bit integer.
        input value range:[0,65535]
        If signed is True, it will be converted to a 16-bit signed integer. Range: [-32768, 32767]
        If signed is False, it will be converted to a 16-bit unsigned integer. Range: [0, 65535]
        '''
        # 范围检查
        if not (0 <= value <= 65535):
            print("Error ConvertToNegative_16bit: Input value exceeds the range [0, 65535].")
        # 转换成 16 位无符号整数
        value &= 0xFFFF  # 将 value 转换成 16 位无符号整数
        if signed:
            if value & 0x8000:  # 检查符号位
                value -= 0x10000  # 如果符号位为 1，表示负数，需要减去 2^16
        return value
    
    def ConvertToNegative_int16_t(value: int) -> int:
        '''
        将输入的整数转换为16位有符号整数。
        输入 value 范围:[0,65535]
        return 范围: [-32768, 32767]
        '''
        '''
        Convert the input integer to a 16-bit signed integer.
        input value range:[0,65535]
        return Range: [-32768, 32767]
        '''
        # 范围检查
        if not (0 <= value <= 65535):
            print("Error ConvertToNegative_int16_t: Input value exceeds the range [0, 65535].")
        # 转换成 8 位无符号整数
        value &= 0xFFFF  # 将 value 转换成 8 位无符号整数
        if value & 0x8000:  # 检查符号位
            value -= 0x10000  # 如果符号位为 1，表示负数，需要减去 2^8
        return value

    def ConvertToNegative_uint16_t(value: int) -> int:
        '''
        将输入的整数转换为16位无符号整数。
        输入 value 范围:[0,65535]
        return 范围: [0, 65535]
        '''
        '''
        Convert the input integer to a 16-bit unsigned integer.
        input value range:[0,65535]
        return Range: [0, 65535]
        '''
        # 范围检查
        if not (0 <= value <= 65535):
            print("Error ConvertToNegative_uint16_t: Input value exceeds the range [0, 65535].")
        # 转换成 8 位无符号整数
        value &= 0xFFFF  # 将 value 转换成 8 位无符号整数
        return value

    def ConvertToNegative_32bit(self, value:int, signed:bool=True):
        '''
        将输入的整数转换为32位整数。
        输入 value 范围:[0,4294967295]
        如果 signed 为 True,则转换为32位有符号整数。
        如果 signed 为 False,则转换为32位无符号整数。
        '''
        '''
        Convert the input integer to a 32-bit integer.
        input value range:[0,4294967295]
        If signed is True, it will be converted to a 32-bit signed integer.
        If signed is False, it will be converted to a 32-bit unsigned integer.
        '''
        # 范围检查
        if not (0 <= value <= 4294967295):
            print("Error ConvertToNegative_32bit: Input value exceeds the range [0, 4294967295].")
        # 转换成 32 位无符号整数
        value &= 0xFFFFFFFF  # 将 value 转换成 32 位无符号整数
        if signed:
            if value & 0x80000000:  # 检查符号位
                value -= 0x100000000  # 如果符号位为 1，表示负数，需要减去 2^32
        return value
    
    def ConvertToNegative_int32_t(value: int) -> int:
        '''
        将输入的整数转换为32位有符号整数。
        输入 value 范围:[0,4294967295]
        return范围: [-2147483648, 2147483647]
        '''
        '''
        Convert the input integer to a 32-bit signed integer.
        input value range:[0,4294967295]
        return Range: [-2147483648, 2147483647].
        '''
        # 范围检查
        if not (0 <= value <= 4294967295):
            print("Error ConvertToNegative_32bit: Input value exceeds the range [0, 4294967295].")
        # 转换成 8 位无符号整数
        value &= 0xFFFFFFFF  # 将 value 转换成 8 位无符号整数
        if value & 0x80000000:  # 检查符号位
            value -= 0x100000000  # 如果符号位为 1，表示负数，需要减去 2^8
        return value

    def ConvertToNegative_uint32_t(value: int) -> int:
        '''
        将输入的整数转换为32位无符号整数。
        输入 value 范围:[0,4294967295]
        return范围: [0, 4294967295]
        '''
        '''
        Convert the input integer to a 32-bit unsigned integer.
        input value range:[0,4294967295]
        return Range: [0, 4294967295].
        '''
        # 范围检查
        if not (0 <= value <= 4294967295):
            print("Error ConvertToNegative_32bit: Input value exceeds the range [0, 4294967295].")
        # 转换成 8 位无符号整数
        value &= 0xFFFFFFFF  # 将 value 转换成 8 位无符号整数
        return value

    def ConvertToList_8bit(self, value: int, signed: bool = True):
        '''
        将输入的整数转换为8位整数列表。
        根据signed参数判断是否将其视为带符号整数。
        超出范围时给出提示。
        '''
        '''
        Convert the input integer into an 8-bit integer list.
        The signed parameter determines whether it is treated as a signed integer.
        A warning will be given if the value exceeds the allowed range.
        '''
        if signed:
            if not -128 <= value <= 127:
                raise OverflowError(f"The input value {value} exceeds the range of an 8-bit signed integer [-128, 127].")
            value = ctypes.c_int8(value).value
            return list(struct.unpack("B", struct.pack(">b", value)))
        else:
            if not 0 <= value <= 255:
                raise OverflowError(f"The input value {value} exceeds the range of an 8-bit unsigned integer [0, 255].")
            return list(struct.unpack("B", struct.pack(">B", value)))
    
    def ConvertToList_int8_t(self, value: int):
        if not -128 <= value <= 127:
            raise OverflowError(f"输入的值 {value} 超出了8位有符号整数的范围 [-128, 127].")
        if value < 0:
            value = (value + 0x100) & 0xFF  # 转换为8位表示
        else:
            value &= 0xFF
        return [value]
    
    def ConvertToList_uint8_t(self, value: int):
        if not 0 <= value <= 255:
            raise OverflowError(f"输入的值 {value} 超出了8位无符号整数的范围 [0, 255].")
        value &= 0xFF
        return [value]

    def ConvertToList_16bit(self, value: int, signed: bool = True):
        '''
        将输入的整数转换为16位整数列表。
        根据signed参数判断是否将其视为带符号整数。
        超出范围时给出提示。
        '''
        '''
        Convert the input integer into a 16-bit integer list.
        The signed parameter determines whether it is treated as a signed integer.
        A warning will be given if the value exceeds the allowed range.
        '''
        if signed:
            if not -32768 <= value <= 32767:
                raise OverflowError(f"The input value {value} exceeds the range of a 16-bit signed integer [-32768, 32767].")
            value = ctypes.c_int16(value).value
            return list(struct.unpack("BB", struct.pack(">h", value)))
        else:
            if not 0 <= value <= 65535:
                raise OverflowError(f"The input value {value} exceeds the range of a 16-bit unsigned integer [0, 65535].")
            return list(struct.unpack("BB", struct.pack(">H", value)))

    def ConvertToList_int16_t(self, value: int):
        if not -32768 <= value <= 32767:
            raise OverflowError(f"输入的值 {value} 超出了16位有符号整数的范围 [-32768, 32767].")
        if value < 0:
            value = (value + 0x10000) & 0xFFFF  # 转换为16位表示
        else:
            value &= 0xFFFF
        # 将结果拆分为两个uint8值
        high_byte = (value >> 8) & 0xFF
        low_byte = value & 0xFF
        return [high_byte, low_byte]
    
    def ConvertToList_uint16_t(self, value: int):
        if not 0 <= value <= 65535:
            raise OverflowError(f"输入的值 {value} 超出了16位无符号整数的范围 [0, 65535].")
        value &= 0xFFFF
        # 将结果拆分为两个uint8值
        high_byte = (value >> 8) & 0xFF
        low_byte = value & 0xFF
        return [high_byte, low_byte]

    def ConvertToList_32bit(self, value: int, signed: bool = True):
        '''
        将输入的整数转换为32位整数列表。
        根据signed参数判断是否将其视为带符号整数。
        超出范围时给出提示。
        '''
        '''
        Convert the input integer into a 32-bit integer list.
        The signed parameter determines whether it is treated as a signed integer.
        A warning will be given if the value exceeds the allowed range.
        '''
        if signed:
            if not -2147483648 <= value <= 2147483647:
                raise OverflowError(f"The input value {value} exceeds the range of a 32-bit signed integer [-2147483648, 2147483647].")
            value = ctypes.c_int32(value).value
            return list(struct.unpack("BBBB", struct.pack(">i", value)))
        else:
            if not 0 <= value <= 4294967295:
                raise OverflowError(f"The input value {value} exceeds the range of a 32-bit unsigned integer [0, 4294967295].")
            return list(struct.unpack("BBBB", struct.pack(">I", value)))

    def ConvertToList_int32_t(self, value: int):
        if not -2147483648 <= value <= 2147483647:
            raise OverflowError(f"输入的值 {value} 超出了32位有符号整数的范围 [-2147483648, 2147483647].")
        if value < 0:
            value = (value + 0x100000000) & 0xFFFFFFFF  # 转换为32位表示
        else:
            value &= 0xFFFFFFFF
        # 将结果拆分为四个uint8值
        byte_3 = (value >> 24) & 0xFF
        byte_2 = (value >> 16) & 0xFF
        byte_1 = (value >> 8) & 0xFF
        byte_0 = value & 0xFF
        return [byte_3, byte_2, byte_1, byte_0]
    
    def ConvertToList_uint32_t(self, value: int):
        if not 0 <= value <= 4294967295:
            raise OverflowError(f"输入的值 {value} 超出了32位无符号整数的范围 [0, 4294967295].")
        value &= 0xFFFFFFFF
        # 将结果拆分为四个uint8值
        byte_3 = (value >> 24) & 0xFF
        byte_2 = (value >> 16) & 0xFF
        byte_1 = (value >> 8) & 0xFF
        byte_0 = value & 0xFF
        return [byte_3, byte_2, byte_1, byte_0]

    def FloatToUint(self,x_float:float, x_min:float, x_max:float, bits:int):
        '''
        浮点数转换为无符号整数
        用在mit模式透传控制单独关节电机驱动器模式
        '''
        '''
        Convert floating point number to unsigned integer
        Used in MIT mode pass-through control for individual joint motor driver mode
        '''
        span:float = x_max - x_min
        offset:float = x_min
        return int((x_float - offset) * (float((1<<bits)-1))/span)
    
    def ConvertBytesToInt(self, bytes:bytearray, first_index:int, second_index:int, byteorder:Literal["little", "big"]='big'):
        '''
        将字节串转换为int类型,默认为大端对齐
        '''
        '''
        Convert a byte string to an int type, with big-endian byte order by default.
        '''
        return int.from_bytes(bytes[first_index:second_index], byteorder=byteorder)


#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 夹爪控制demo

from typing import (
    Optional,
)
import time
from piper_sdk import *

def enable_fun(piper:C_PiperInterface):
    '''
    使能机械臂并检测使能状态,尝试5s,如果使能超时则退出程序
    '''
    enable_flag = False
    # 设置超时时间（秒）
    timeout = 5
    # 记录进入循环前的时间
    start_time = time.time()
    elapsed_time_flag = False
    while not (enable_flag):
        elapsed_time = time.time() - start_time
        print("--------------------")
        enable_flag = piper.GetArmLowSpdInfoMsgs().motor_1.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_2.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_3.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_4.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_5.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_6.foc_status.driver_enable_status
        print("使能状态:",enable_flag)
        piper.EnableArm(7)
        piper.GripperCtrl(0,1000,0x01, 0)
        print("--------------------")
        # 检查是否超过超时时间
        if elapsed_time > timeout:
            print("超时....")
            elapsed_time_flag = True
            enable_flag = True
            break
        time.sleep(1)
        pass
    if(elapsed_time_flag):
        print("程序自动使能超时,退出程序")
        exit(0)

if __name__ == "__main__":
    piper = C_PiperInterface("can0")
    piper.ConnectPort()
    piper.EnableArm(7)
    enable_fun(piper=piper)
    piper.GripperCtrl(0,1000,0x01, 0)
    range = 0
    count = 0
    while True:
        print(piper.GetArmGripperMsgs())
        import time
        count  = count + 1
        # print(count)
        if(count == 0):
            print("1-----------")
            range = 0
        elif(count == 300):
            print("2-----------")
            range = 50 * 1000 # 50mm
        elif(count == 600):
            print("1-----------")
            range = 0
            count = 0
        piper.GripperCtrl(abs(range), 1000, 0x01, 0)
        
        time.sleep(0.005)
        pass


#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)

class ArmMsgSearchMotorMaxAngleSpdAccLimit:
    '''
    msg_v1_transmit
    
    查询电机角度/最大速度/最大加速度限制指令

    CAN ID:
        0x472

    Args:
        motor_num: 关节电机序号,1-6
        search_content: 查询内容,0x01-查询电机角度/最大速度,0x02-查询电机最大加速度限制

    位描述:
    
        :Byte 0 motor_num: uint8, 关节电机序号。
                值域 1-6,1-6 代表关节驱动器序号
        :Byte 1 search_content: uint8, 查询内容。
                0x01 : 查询电机角度/最大速度
                0x02 : 查询电机最大加速度限制
    '''
    '''
    msg_v1_transmit
    
    Motor Angle/Max Speed/Max Acceleration Limit Query Command

    CAN ID:
        0x472

    Args:
        motor_num: Motor joint number.
        search_content: Query content.

    Bit Description:

        Byte 0: uint8, motor joint number.
            Value range: 1-6.
                1-6: Represent joint driver numbers.
        Byte 1: uint8, query content.
            0x01: Query motor angle/max speed.
            0x02: Query motor max acceleration limit.
    '''
    def __init__(self, 
                 motor_num: Literal[1, 2, 3, 4, 5, 6] = 1,
                 search_content: Literal[0x01, 0x02] = 0x01):
        if motor_num not in [1, 2, 3, 4, 5, 6, 7]:
            raise ValueError(f"'motor_num' Value {motor_num} out of range [1, 2, 3, 4, 5, 6, 7]")
        if search_content not in [0x01, 0x02]:
            raise ValueError(f"'search_content' Value {search_content} out of range [0x01, 0x02]")
        self.motor_num = motor_num
        self.search_content = search_content

    def __str__(self):
        return (f"ArmMsgSearchMotorMaxAngleSpdAccConfig(\n"
                f"  motor_num: {self.motor_num },\n"
                f"  search_content: {self.search_content },\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
import math
class ArmMsgFeedbackCurrentEndVelAccParam:
    '''
    msg_v1_feedback
    
    反馈当前末端速度/加速度参数

    CAN ID:
        0x478

    Args:
        end_max_linear_vel: 末端最大线速度
        end_max_angular_vel: 末端最大角速度
        end_max_linear_acc: 末端最大线加速度
        end_max_angular_acc: 末端最大角加速度
    
    位描述:

        Byte 0: 末端最大线速度 H, uint16, 单位 0.001m/s
        Byte 1: 末端最大线速度 L,
        Byte 2: 末端最大角速度 H, uint16, 单位 0.001rad/s
        Byte 3: 末端最大角速度 L,
        Byte 4: 末端最大线加速度 H, uint16, 单位 0.001m/s^2
        Byte 5: 末端最大线加速度 L
        Byte 6: 末端最大角加速度 H, uint16, 单位 0.001rad/s^2
        Byte 7: 末端最大角加速度 L
    '''
    '''
    msg_v1_feedback
    
    Feedback of Current End-Effector Speed/Acceleration Parameters

    CAN ID: 
        0x478

    Args:
        end_max_linear_vel: Maximum linear velocity of the end-effector.
        end_max_angular_vel: Maximum angular velocity of the end-effector.
        end_max_linear_acc: Maximum linear acceleration of the end-effector.
        end_max_angular_acc: Maximum angular acceleration of the end-effector.
    
    Bit Description:

        Byte 0: Maximum Linear Velocity (High Byte), uint16, unit: 0.001 m/s
        Byte 1: Maximum Linear Velocity (Low Byte)
        Byte 2: Maximum Angular Velocity (High Byte), uint16, unit: 0.001 rad/s
        Byte 3: Maximum Angular Velocity (Low Byte)
        Byte 4: Maximum Linear Acceleration (High Byte), uint16, unit: 0.001 m/s²
        Byte 5: Maximum Linear Acceleration (Low Byte)
        Byte 6: Maximum Angular Acceleration (High Byte), uint16, unit: 0.001 rad/s²
        Byte 7: Maximum Angular Acceleration (Low Byte)
    '''
    def __init__(self, 
                 end_max_linear_vel: int = 0, 
                 end_max_angular_vel: int = 0, 
                 end_max_linear_acc: int = 0,
                 end_max_angular_acc: int = 0
                 ):
        self.end_max_linear_vel = end_max_linear_vel
        self.end_max_angular_vel = end_max_angular_vel
        self.end_max_linear_acc = end_max_linear_acc
        self.end_max_angular_acc = end_max_angular_acc

    def __str__(self):
        return (f"ArmMsgFeedbackCurrentEndVelAccParam(\n"
                f"  end_max_linear_vel: {self.end_max_linear_vel}, {self.end_max_linear_vel*0.001:.3f}m/s\n"
                f"  end_max_angular_vel: {self.end_max_angular_vel}, {self.end_max_angular_vel*0.001:.3f}rad/s,\n"
                f"  end_max_linear_acc: {self.end_max_linear_acc }, {self.end_max_linear_acc*0.001:.3f}m/s^2,\n"
                f"  end_max_angular_acc: {self.end_max_angular_acc}, {self.end_max_angular_acc*0.001:.3f}rad/s^2\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
import math
from typing_extensions import (
    Literal,
)
class ArmHighSpdFeedback:
    '''
    msg_v2_feedback
    
    驱动器信息高速反馈 0x5

    节点 ID:
        0x1~0x06
    
    CAN ID:
        0X251~0x256

    Args:
        can_id: 当前canid,用来代表关节序号
        motor_speed: 电机当前转速
        current: 电机当前电流
        pos: 电机当前位置
        effort: 经过固定系数转换的力矩,单位0.001N/m
    
    位描述:

        Byte 0: 转速高八位, int16, 电机当前转速 单位: 0.001rad/s
        Byte 1: 转速低八位
        Byte 2: 电流高八位, uint16, 电机当前电流 单位: 0.001A
        Byte 3: 电流低八位
        Byte 4: 位置最高位, int32, 电机当前位置 单位: rad
        Byte 5: 位置次高位
        Byte 6: 位置次低位
        Byte 7: 位置最低位
    '''
    '''
    msg_v2_feedback
    
    High-Speed Feedback of Drive Information 0x5

    Node ID:
        0x1~0x06

    CAN ID:
        0x251~0x256

    Args:
        can_id: Current CAN ID, used to represent the joint number.
        motor_speed: Motor Speed.
        current: Motor Current.
        pos: Motor Position.
        effort: Torque converted using a fixed coefficient, with a unit of 0.001 N/m.

    
    Bit Description:

        Byte 0: Motor Speed (High Byte), int16, unit: 0.001rad/s
        Byte 1: Motor Speed (Low Byte)
        Byte 2: Motor Current (High Byte), uint16, unit: 0.001A
        Byte 3: Motor Current (Low Byte)
        Byte 4: Motor Position (Most Significant Byte), int32, unit: rad
        Byte 5: Motor Position (Second Most Significant Byte)
        Byte 6: Motor Position (Second Least Significant Byte)
        Byte 7: Motor Position (Least Significant Byte)
    '''
    def __init__(self, 
                 can_id: Literal[0x000, 0x251, 0x252, 0x253, 0x254, 0x255, 0x256] = 0,
                 motor_speed: int = 0, 
                 current: int = 0, 
                 pos: int = 0,
                 effort: float = 0
                 ):
        if can_id not in [0x000, 0x251, 0x252, 0x253, 0x254, 0x255, 0x256]:
            raise ValueError(f"'can_id' Value {can_id} out of range [0x000, 0x251, 0x252, 0x253, 0x254, 0x255, 0x256]")
        self.COEFFICIENT_1_3 = 1.18125
        self.COEFFICIENT_4_6 = 0.95844
        self.can_id = can_id
        self.motor_speed = motor_speed
        self.current = current
        self.pos = pos
        self.effort = effort
    
    def cal_effort(self, current: int = None)-> float:
        current_ = 0
        if(current is None):
            current_ = self.current
        elif(isinstance(current, (int, float))):
            current_ = current
        else:
            raise TypeError(f"current {current} is not 'int' or 'float'.")
        if(self.can_id in [0x251, 0x252, 0x253]):
            self.effort = current_ * self.COEFFICIENT_1_3
        elif(self.can_id in [0x254, 0x255, 0x256]):
            self.effort = current_ * self.COEFFICIENT_4_6
        return self.effort

    def __str__(self):
        return (f"ArmHighSpdFeedback(\n"
                f"  can_id: {hex(self.can_id)},\n"
                f"  motor_speed: {self.motor_speed}, {self.current*0.001} rad/s,\n"
                f"  current: {self.current}, {self.current*0.001}A\n"
                f"  pos: {self.pos} rad\n"
                f"  effort: {self.effort}, {self.effort*0.001}N/m\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)
class ArmMsgMotionCtrl_1():
    '''
    msg_v2_transmit
    
    机械臂运动控制指令1
    
    CAN ID:
        0x150
    
    Args:
        emergency_stop: 快速急停
        track_ctrl: 轨迹指令
        grag_teach_ctrl: 拖动示教指令
    
    位描述:
    
        Byte 0: 快速急停     uint8    0x00 无效
                                    0x01 快速急停
                                    0x02 恢复
        Byte 1: 轨迹指令     uint8    0x00 关闭
                                    0x01 暂停当前规划
                                    0x02 继续当前轨迹
                                    0x03 清除当前轨迹
                                    0x04 清除所有轨迹
                                    0x05 获取当前规划轨迹
                                    0x06 终止执行
                                    0x07 轨迹传输
                                    0x08 轨迹传输结束
        Byte 2: 拖动示教指令 uint8     0x00 关闭
                                    0x01 开始示教记录（进入拖动示教模式）
                                    0x02 结束示教记录（退出拖动示教模式）
                                    0x03 执行示教轨迹（拖动示教轨迹复现）
                                    0x04 暂停执行
                                    0x05 继续执行（轨迹复现继续）
                                    0x06 终止执行
                                    0x07 运动到轨迹起点
        Byte 3: 轨迹索引    uint8     标记刚才传输的轨迹点为第N个轨迹点
                                    N=0~255
                                    主控收到后会应答0x476 byte0 = 0x50 ;byte 2=N(详见0x476 )未收到应答需要重传
        Byte 4: NameIndex_H uint16   当前轨迹包名称索引,由NameIndex和crc组成(应答0x477 byte0=03)
        Byte 5: NameIndex_L
        Byte 6: crc16_H     uint16  
        Byte 7: crc16_L
    '''
    '''
    msg_v2_transmit
    
    Robotic Arm Motion Control Command 1

    CAN ID:
        0x150

    Args:
        emergency_stop: Emergency stop command.
        track_ctrl: Trajectory control command.
        grag_teach_ctrl: Drag teach command.

    Bit Descriptions:

        Byte 0: emergency_stop: uint8, emergency stop control.
            0x00: Invalid.
            0x01: Activate emergency stop.
            0x02: Resume from emergency stop.

        Byte 1: track_ctrl: uint8, trajectory control instructions.
            0x00: Disable.
            0x01: Pause current planning.
            0x02: Resume current trajectory.
            0x03: Clear current trajectory.
            0x04: Clear all trajectories.
            0x05: Get the current planned trajectory.
            0x06: Terminate execution.
            0x07: Trajectory transfer.
            0x08: End trajectory transfer.

        Byte 2: grag_teach_ctrl: uint8, drag teach control.
            0x00: Disable.
            0x01: Start teaching record (enter drag teach mode).
            0x02: End teaching record (exit drag teach mode).
            0x03: Execute the teaching trajectory (reproduce drag teaching trajectory).
            0x04: Pause execution.
            0x05: Continue execution (resume trajectory reproduction).
            0x06: Terminate execution.
            0x07: Move to the starting point of the trajectory.

        Byte 3: trajectory_index: uint8, mark the transmitted trajectory point as the Nth trajectory point.
            N = 0~255:
            The controller responds with CAN ID: 0x476, Byte 0 = 0x50, Byte 2 = N. If no response is received, retransmission is required.

        Byte 4-5: NameIndex_H/L: uint16, current trajectory packet name index, composed of NameIndex and CRC.
            Response on CAN ID: 0x477, Byte 0 = 0x03.

        Byte 6-7: crc16_H/L: uint16, CRC checksum for validation.
    '''
    def __init__(self, 
                 emergency_stop: Literal[0x00, 0x01, 0x02] = 0, 
                 track_ctrl: Literal[0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08] = 0, 
                 grag_teach_ctrl: Literal[0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07] = 0):
        # 检查 emergency_stop 是否在有效范围内
        if emergency_stop not in [0x00, 0x01, 0x02]:
            raise ValueError(f"'emergency_stop' Value {emergency_stop} out of range [0x00, 0x01, 0x02]")
        
        # 检查 track_ctrl 是否在有效范围内
        if track_ctrl not in [0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08]:
            raise ValueError(f"'track_ctrl' Value {track_ctrl} out of range [0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08]")
        
        # 检查 grag_teach_ctrl 是否在有效范围内
        if grag_teach_ctrl not in [0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07]:
            raise ValueError(f"'grag_teach_ctrl' Value {grag_teach_ctrl} out of range [0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07]")
        self.emergency_stop = emergency_stop
        self.track_ctrl = track_ctrl
        self.grag_teach_ctrl = grag_teach_ctrl

    def __str__(self):
        dict_ = [
            (" emergency_stop ", self.emergency_stop),
            (" track_ctrl ", self.track_ctrl),
            (" grag_teach_ctrl ", self.grag_teach_ctrl)
        ]

        # 生成格式化字符串，保留三位小数
        formatted_ = "\n".join([f"{name}: {value}" for name, value in dict_])
        
        return f"ArmMsgMotionCtrl_1:\n{formatted_}"
    
    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-

class ArmMsgJointCtrl():
    '''
    msg_v1_transmit
    
    机械臂关节控制,单位0.001度
    
    CAN ID:
        0x155,0x156,0x157
    
    Args:
        joint_1: joint_1角度
        joint_2: joint_2角度
        joint_3: joint_3角度
        joint_4: joint_4角度
        joint_5: joint_5角度
        joint_6: joint_6角度
    '''
    '''
    msg_v1_transmit
    
    Robotic Arm Joint Control (Unit: 0.001°)

    CAN IDs:
        0x155, 0x156, 0x157
    
    Args:
        joint_1: The target angle of joint 1 in 0.001°.
        joint_2: The target angle of joint 2 in 0.001°.
        joint_3: The target angle of joint 3 in 0.001°.
        joint_4: The target angle of joint 4 in 0.001°.
        joint_5: The target angle of joint 5 in 0.001°.
        joint_6: The target angle of joint 6 in 0.001°.
    '''
    def __init__(self, 
                 joint_1: int = 0, 
                 joint_2: int = 0, 
                 joint_3: int = 0, 
                 joint_4: int = 0, 
                 joint_5: int = 0, 
                 joint_6: int = 0):
        self.joint_1 = joint_1
        self.joint_2 = joint_2
        self.joint_3 = joint_3
        self.joint_4 = joint_4
        self.joint_5 = joint_5
        self.joint_6 = joint_6

    def __str__(self):
        # 将角度乘以0.001，并保留三位小数
        joint_angles = [
            ("Joint_1", self.joint_1 * 0.001),
            ("Joint_2", self.joint_2 * 0.001),
            ("Joint_3", self.joint_3 * 0.001),
            ("Joint_4", self.joint_4 * 0.001),
            ("Joint_5", self.joint_5 * 0.001),
            ("Joint_6", self.joint_6 * 0.001)
        ]

        # 生成格式化字符串，保留三位小数
        formatted_angles = "\n".join([f"{name}: {angle:.3f}" for name, angle in joint_angles])
        
        return f"ArmMsgJointCtrl:\n{formatted_angles}"
    
    def __repr__(self):
        return self.__str__()


#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 设置机械臂重置，需要在mit或者示教模式切换为位置速度控制模式时执行
# 使用后需要reset，并重新使能两次

from typing import (
    Optional,
)
import time
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface_V2()
    piper.ConnectPort()
    piper.MotionCtrl_1(0x01,0,0)



#!/usr/bin/env python3
# -*-coding:utf8-*-

class ArmMsgGripperFeedBack:
    '''
    msg_v1_feedback
    
    夹爪反馈消息
    
    CAN ID:
        0x2A8
    
    Args:
        grippers_angle: 夹爪角度，以整数表示。
        grippers_effort: 夹爪扭矩，以整数表示。
        status_code: 夹爪状态码，以整数表示。
    
    位描述:

        Byte 0: 夹爪行程最高位, int32, 单位 0.001mm
        Byte 1: 
        Byte 2: 
        Byte 3: 
        Byte 4: 夹爪扭矩 H, int16, 单位 0.001N/m
        Byte 5: 夹爪扭矩 L
        Byte 6: 状态码, uint8
            bit[0]      电源电压是否过低(0:正常 1:过低)
            bit[1]      电机是否过温(0:正常 1:过温)
            bit[2]      驱动器是否过流(0:正常 1:过流)
            bit[3]      驱动器是否过温(0:正常 1:过温)
            bit[4]      传感器状态(0:正常 1:异常)
            bit[5]      驱动器错误状态(0:正常 1:错误)
            bit[6]      驱动器使能状态(1:使能 0:失能)
            bit[7]      回零状态(0:没有回零 1:已经回零,或已经回过零)
        Byte 7: 保留
    '''
    '''
    msg_v1_feedback
    
    Gripper Feedback Message

    CAN ID:
        0x2A8
    
    Args:
        grippers_angle: The angle of the gripper, represented as an integer.
        grippers_effort: The torque of the gripper, represented as an integer.
        status_code: The status code of the gripper, represented as an integer.
    
    Bit Definitions:

        Byte Definitions:
        Byte 0: Gripper Stroke (Most Significant Byte), int32, unit: 0.001 mm
        Byte 1: Gripper Stroke (Second Most Significant Byte)
        Byte 2: Gripper Stroke (Second Least Significant Byte)
        Byte 3: Gripper Stroke (Least Significant Byte)
        Byte 4: Gripper Torque (High Byte), int16, unit: 0.001 N·m
        Byte 5: Gripper Torque (Low Byte)
        Byte 6: Status Code, uint8:
            bit[0]: Power voltage low (0: Normal, 1: Low)
            bit[1]: Motor over-temperature (0: Normal, 1: Over-temperature)
            bit[2]: Driver over-current (0: Normal, 1: Over-current)
            bit[3]: Driver over-temperature (0: Normal, 1: Over-temperature)
            bit[4]: Sensor status (0: Normal, 1: Abnormal)
            bit[5]: Driver error status (0: Normal, 1: Error)
            bit[6]: Driver enable status (1: Enabled, 0: Disabled)
            bit[7]: Zeroing status (0: Not zeroed, 1: Zeroed or previously zeroed)
        Byte 7: Reserved
    '''
    def __init__(self, 
                 grippers_angle: int = 0, 
                 grippers_effort: int = 0, 
                 status_code: int = 0):
        self.grippers_angle = grippers_angle
        self.grippers_effort = grippers_effort
        self._status_code = status_code
        self.foc_status = self.FOC_Status()
    
    class FOC_Status:
        def __init__(self):
            self.voltage_too_low  = False
            self.motor_overheating = False
            self.driver_overcurrent = False
            self.driver_overheating = False
            self.sensor_status = False
            self.driver_error_status = False
            self.driver_enable_status = False
            self.homing_status  = False
        def __str__(self): 
            return (f"    voltage_too_low : {self.voltage_too_low}\n"
                    f"    motor_overheating: {self.motor_overheating}\n"
                    f"    driver_overcurrent: {self.driver_overcurrent}\n"
                    f"    driver_overheating: {self.driver_overheating}\n"
                    f"    sensor_status: {self.sensor_status}\n"
                    f"    driver_error_status: {self.driver_error_status}\n"
                    f"    driver_enable_status: {self.driver_enable_status}\n"
                    f"    homing_status: {self.homing_status}\n"
                    )
    @property
    def status_code(self):
        return self._status_code

    @status_code.setter
    def status_code(self, value: int):
        if not (0 <= value < 2**8):
            raise ValueError("status_code must be an 8-bit integer between 0 and 255.")
        self._status_code = value
        # Update foc_status based on the status_code bits
        self.foc_status.voltage_too_low = bool(value & (1 << 0))
        self.foc_status.motor_overheating = bool(value & (1 << 1))
        self.foc_status.driver_overcurrent = bool(value & (1 << 2))
        self.foc_status.driver_overheating = bool(value & (1 << 3))
        self.foc_status.sensor_status = bool(value & (1 << 4))
        self.foc_status.driver_error_status = bool(value & (1 << 5))
        self.foc_status.driver_enable_status = bool(value & (1 << 6))
        self.foc_status.homing_status = bool(value & (1 << 7))
    
    def __str__(self):
        return (f"ArmMsgGripperFeedBack(\n"
                f"  grippers_angle: {self.grippers_angle}, {self.grippers_angle * 0.001:.3f},\n"
                f"  grippers_effort: {self.grippers_effort} \t {self.grippers_effort * 0.001:.3f},\n"
                f"  status_code: \n{self.foc_status}\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-

class ArmMsgCrashProtectionRatingFeedback:
    '''
    msg_v1_feedback
    
    碰撞防护等级反馈指令
    
    CAN ID: 
        0x47B
    
    Args:
        joint_1_protection_level: 1号关节碰撞防护等级
        joint_2_protection_level: 2号关节碰撞防护等级
        joint_3_protection_level: 3号关节碰撞防护等级
        joint_4_protection_level: 4号关节碰撞防护等级
        joint_5_protection_level: 5号关节碰撞防护等级
        joint_6_protection_level: 6号关节碰撞防护等级
    
    设定值 : 0~8

    等级 0 代表不检测碰撞； 6个关节可以独立设置

    位描述:

        Byte 0: 1 号关节碰撞防护等级, uint8
        Byte 1: 2 号关节碰撞防护等级, uint8
        Byte 2: 3 号关节碰撞防护等级, uint8
        Byte 3: 4 号关节碰撞防护等级, uint8
        Byte 4: 5 号关节碰撞防护等级, uint8
        Byte 5: 6 号关节碰撞防护等级, uint8
        Byte 6: 保留
        Byte 7: 保留
    '''
    
    '''
    msg_v1_feedback
    
    Get the collision protection level feedback for each joint.

    CAN ID: 
        0x47B

    Args:
        joint_1_protection_level (int): Collision protection level for joint 1 (0-8)
        joint_2_protection_level (int): Collision protection level for joint 2 (0-8)
        joint_3_protection_level (int): Collision protection level for joint 3 (0-8)
        joint_4_protection_level (int): Collision protection level for joint 4 (0-8)
        joint_5_protection_level (int): Collision protection level for joint 5 (0-8)
        joint_6_protection_level (int): Collision protection level for joint 6 (0-8)

    Level Description:
        0: No collision detection
        1-8: Collision detection thresholds increase (higher values represent more sensitive thresholds)

    Byte Description:

        Byte 0: Collision protection level for joint 1, uint8
        Byte 1: Collision protection level for joint 2, uint8
        Byte 2: Collision protection level for joint 3, uint8
        Byte 3: Collision protection level for joint 4, uint8
        Byte 4: Collision protection level for joint 5, uint8
        Byte 5: Collision protection level for joint 6, uint8
        Byte 6: Reserved
        Byte 7: Reserved
    '''
    def __init__(self, 
                 joint_1_protection_level: int = 0, 
                 joint_2_protection_level: int = 0, 
                 joint_3_protection_level: int = 0,
                 joint_4_protection_level: int = 0,
                 joint_5_protection_level: int = 0,
                 joint_6_protection_level: int = 0
                 ):
        self.joint_1_protection_level = joint_1_protection_level
        self.joint_2_protection_level = joint_2_protection_level
        self.joint_3_protection_level = joint_3_protection_level
        self.joint_4_protection_level = joint_4_protection_level
        self.joint_5_protection_level = joint_5_protection_level
        self.joint_6_protection_level = joint_6_protection_level

    def __str__(self):
        '''
        返回对象的字符串表示，用于打印。
        '''
        '''
        Return the string representation of the object for printing.
        '''
        return (f"ArmMsgCrashProtectionRatingFeedback(\n"
                f"  joint_1_protection_level: {self.joint_1_protection_level},\n"
                f"  joint_2_protection_level: {self.joint_2_protection_level},\n"
                f"  joint_3_protection_level: {self.joint_3_protection_level},\n"
                f"  joint_4_protection_level: {self.joint_4_protection_level},\n"
                f"  joint_5_protection_level: {self.joint_5_protection_level},\n"
                f"  joint_6_protection_level: {self.joint_6_protection_level}\n"
                f")")

    def __repr__(self):
        '''
        返回对象的正式字符串表示，通常用于调试。

        :return: 对象的字符串表示，与 __str__ 相同。
        '''
        '''
        Return the formal string representation of the object, typically used for debugging.
        
        :return: The string representation of the object, identical to `__str__`.
        '''
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# V2版本sdk
# 机械臂 设置全部关节限位、关节最大速度、关节加速度为默认值： 0x02

from typing import (
    Optional,
)
import time
from piper_sdk import *

def enable_fun(piper:C_PiperInterface_V2, enable:bool):
    '''
    使能机械臂并检测使能状态,尝试5s,如果使能超时则退出程序
    '''
    enable_flag = False
    loop_flag = False
    # 设置超时时间（秒）
    timeout = 5
    # 记录进入循环前的时间
    start_time = time.time()
    elapsed_time_flag = False
    while not (loop_flag):
        elapsed_time = time.time() - start_time
        print(f"--------------------")
        enable_list = []
        enable_list.append(piper.GetArmLowSpdInfoMsgs().motor_1.foc_status.driver_enable_status)
        enable_list.append(piper.GetArmLowSpdInfoMsgs().motor_2.foc_status.driver_enable_status)
        enable_list.append(piper.GetArmLowSpdInfoMsgs().motor_3.foc_status.driver_enable_status)
        enable_list.append(piper.GetArmLowSpdInfoMsgs().motor_4.foc_status.driver_enable_status)
        enable_list.append(piper.GetArmLowSpdInfoMsgs().motor_5.foc_status.driver_enable_status)
        enable_list.append(piper.GetArmLowSpdInfoMsgs().motor_6.foc_status.driver_enable_status)
        if(enable):
            enable_flag = all(enable_list)
            piper.EnableArm(7)
            piper.GripperCtrl(0,1000,0x01, 0)
        else:
            enable_flag = any(enable_list)
            piper.DisableArm(7)
            piper.GripperCtrl(0,1000,0x02, 0)
        print(f"使能状态: {enable_flag}")
        print(f"--------------------")
        if(enable_flag == enable):
            loop_flag = True
            enable_flag = True
        else: 
            loop_flag = False
            enable_flag = False
        # 检查是否超过超时时间
        if elapsed_time > timeout:
            print(f"超时....")
            elapsed_time_flag = True
            enable_flag = False
            loop_flag = True
            break
        time.sleep(0.5)
    resp = enable_flag
    print(f"Returning response: {resp}")
    return resp

if __name__ == "__main__":
    piper = C_PiperInterface_V2("can0")
    piper.ConnectPort()
    piper.EnableArm(7)
    # enable_fun(piper=piper, enable=True)
    piper.ArmParamEnquiryAndConfig(0x01,0x02,0,0,0x02)
    while True:
        piper.SearchAllMotorMaxAngleSpd()
        print(piper.GetAllMotorAngleLimitMaxSpd())
        time.sleep(0.01)


#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)

class ArmMsgMotorEnableDisableConfig:
    '''
    msg_v2_transmit
    
    电机使能/失能设置指令
    
    CAN ID:
        0x471
    
    Args:
        motor_num: 电机序号[1,7],7代表所有电机
        enable_flag: 使能标志位,0x01-失能;0x02-使能
    
    位描述:
    
        Byte 0: uint8, 关节电机序号。
                值域 1-7:1-6 代表关节驱动器序号,7代表夹爪电机,FF代表全部关节电机(包含夹爪)
        Byte 1: uint8, 使能/失能。
                0x01 : 失能
                0x02 : 使能
    '''
    '''
    msg_v2_transmit
    
    Motor Enable/Disable Command

    CAN ID:
        0x471

    Args:
        motor_num: Motor index [1, 7], where 7 represents all motors.
        enable_flag: Enable flag, 0x01 for disable, 0x02 for enable.

    Bit Description:

        Byte 0:
            motor_num, uint8, motor index.
            Range 1-7:
                1-6: Represents joint motor index.
                7: Represents gripper motor.
                0xFF: Represents all joint motors (including gripper).
        Byte 1:
            enable_flag, uint8, enable/disable.
            0x01: Disable.
            0x02: Enable.
    '''
    def __init__(self, 
                 motor_num: Literal[1, 2, 3, 4, 5, 6, 7, 0xFF] = 0xFF,
                 enable_flag: Literal[0x01, 0x02] = 0x01):
        if motor_num not in [1, 2, 3, 4, 5, 6, 7, 0xFF]:
            raise ValueError(f"'motor_num' Value {motor_num} out of range [1, 2, 3, 4, 5, 6, 7, 0xFF]")
        if enable_flag not in [0x01, 0x02]:
            raise ValueError(f"'enable_flag' Value {enable_flag} out of range [0x01, 0x02]")
        self.motor_num = motor_num
        self.enable_flag = enable_flag

    def __str__(self):
        return (f"ArmMsgMotorEnableDisableConfig(\n"
                f"  motor_num: {self.motor_num },\n"
                f"  enable_flag: {self.enable_flag },\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 夹爪设定零点demo

import time
from piper_sdk import *

if __name__ == "__main__":
    piper = C_PiperInterface("can0")
    piper.ConnectPort()
    piper.EnableArm(7)
    piper.GripperCtrl(0,1000,0x00, 0)
    time.sleep(1.5)
    piper.GripperCtrl(0,1000,0x00, 0xAE)



from setuptools import setup, find_packages # 导入setuptools打包工具
# import setuptools_scm

setup(
    name='piper_sdk',
    version='0.2.18',    # 包版本号，便于维护版本
    # use_scm_version=True,  # 启用 setuptools_scm 来动态获取版本号
    # setup_requires=['setuptools>=40.0', 'setuptools_scm'],  # 指定需要的构建工具
    setup_requires=['setuptools>=40.0'], 
    long_description=open('./piper_sdk/DESCRIPTION.MD').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/agilexrobotics/piper_sdk',  # 主页链接
    license='MIT License',  # 这里显式指定 License
    packages=find_packages(include=['piper_sdk', 'piper_sdk.*']),
    include_package_data=True,
    package_data={
        '': ['LICENSE','*.sh','*.MD'],  # 指定包含的文件
        'piper_sdk/asserts': ['*'],  # 包括 asserts 文件夹下的所有文件
    },
    install_requires=[
        'python-can>=3.3.4',
    ],
    entry_points={
    },
    author='RosenYin',  # 作者
    author_email='yinruocheng321@gmail.com',
    description='A sdk to control Agilex piper arm',   #包的简述
    platforms=['Linux'],
    # url="https://github.com/agilexrobotics/piper_sdk",
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',    #对python的最低版本要求,18.04及以上
    project_urls={
        'Repository': 'https://github.com/agilexrobotics/piper_sdk',
        'ChangeLog': 'https://github.com/agilexrobotics/piper_sdk/blob/master/CHANGELOG.MD',
    },
)




#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 设置机械臂为主动臂，直接设置即可
from typing import (
    Optional,
)
from piper_sdk import *
import time

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface_V2()
    piper.ConnectPort()
    piper.MasterSlaveConfig(0xFA, 0, 0, 0)



#!/usr/bin/env python3
# -*-coding:utf8-*-
from typing_extensions import (
    Literal,
)

class ArmMsgMasterSlaveModeConfig:
    '''
    msg_v1_transmit
    
    随动主从模式设置指令
    
    CAN ID:
        0x470
    
    Args:
        linkage_config: 联动设置指令
        feedback_offset: 反馈指令偏移值
        ctrl_offset: 控制指令偏移值
        linkage_offset: 联动模式控制目标地址偏移值
    
    位描述:
    
        Byte 0 linkage_config: uint8, 联动设置指令。
                                0x00 无效
                                0xFA 设置为示教输入臂
                                0xFC 设置为运动输出臂
        Byte 1 feedback_offset: uint8, 反馈指令偏移值。
                                0x00 : 不偏移/恢复默认
                                0x10 ：反馈指令基 ID 由 2Ax偏移为 2Bx
                                0x20 ：反馈指令基 ID 由 2Ax偏移为 2Cx
        Byte 2 ctrl_offset: uint8, 控制指令偏移值。
                                0x00 : 不偏移/恢复默认
                                0x10 ：控制指令基 ID 由 15x偏移为 16x
                                0x20 ：控制指令基 ID 由 15x偏移为 17x
        Byte 3 linkage_offset: uint8, 联动模式控制目标地址偏移值。
                                0x00 : 不偏移/恢复默认
                                0x10 : 控制目标地址基 ID由 15x 偏移为 16x
                                0x20 : 控制目标地址基 ID由 15x 偏移为 17x
    '''
    '''
    msg_v1_transmit
    
    Follow Master-Slave Mode Setting Command

    CAN ID:
        0x470

    Args:
        linkage_config: Linkage setting command.
        feedback_offset: Offset value for feedback instructions.
        ctrl_offset: Offset value for control instructions.
        linkage_offset: Offset value for linkage mode control target address.

    Bit Descriptions:

        Byte 0 linkage_config: uint8, linkage setting command.
            0x00: Invalid.
            0xFA: Set as teaching input arm.
            0xFC: Set as motion output arm.

        Byte 1 feedback_offset: uint8, feedback instruction offset value.
            0x00: No offset/restore default.
            0x10: Feedback instruction base ID shifted from 2Ax to 2Bx.
            0x20: Feedback instruction base ID shifted from 2Ax to 2Cx.

        Byte 2 ctrl_offset: uint8, control instruction offset value.
            0x00: No offset/restore default.
            0x10: Control instruction base ID shifted from 15x to 16x.
            0x20: Control instruction base ID shifted from 15x to 17x.

        Byte 3 linkage_offset: uint8, offset value for the linkage mode control target address.
            0x00: No offset/restore default.
            0x10: Control target address base ID shifted from 15x to 16x.
            0x20: Control target address base ID shifted from 15x to 17x.
    '''
    def __init__(self, 
                 linkage_config: Literal[0x00, 0xFA, 0xFC] = 0x00,
                 feedback_offset: Literal[0x00, 0x10, 0x20] = 0x00,
                 ctrl_offset: Literal[0x00, 0x10, 0x20] = 0x00,
                 linkage_offset: Literal[0x00, 0x10, 0x20] = 0x00):
        if linkage_config not in [0x00, 0xFA, 0xFC]:
            raise ValueError(f"'linkage_config' Value {linkage_config} out of range [0x00, 0xFA, 0xFC]")
        if feedback_offset not in [0x00, 0x10, 0x20]:
            raise ValueError(f"'feedback_offset' Value {feedback_offset} out of range [0x00, 0x10, 0x20]")
        if ctrl_offset not in [0x00, 0x10, 0x20]:
            raise ValueError(f"'ctrl_offset' Value {ctrl_offset} out of range [0x00, 0x10, 0x20]")
        if linkage_offset not in [0x00, 0x10, 0x20]:
            raise ValueError(f"'linkage_offset' Value {linkage_offset} out of range [0x00, 0x10, 0x20]")
        self.linkage_config = linkage_config
        self.feedback_offset = feedback_offset
        self.ctrl_offset = ctrl_offset
        self.linkage_offset = linkage_offset

    def __str__(self):
        return (f"ArmMsgMasterSlaveModeConfig(\n"
                f"  linkage_config: {self.linkage_config },\n"
                f"  feedback_offset: {self.feedback_offset },\n"
                f"  ctrl_offset: {self.ctrl_offset },\n"
                f"  linkage_offset: {self.linkage_offset}\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-

from enum import Enum, auto

class CanIDPiper(Enum):
    '''
    机械臂can id
    '''
    '''
    piper's can_ids
    '''
    #---------------------------------feedback------------------------------------#
    # 主动反馈指令，可设置整体偏移为 0x2B1~0x2B8或 0x2C1~0x2C8，详见指令0x470
    ARM_STATUS_FEEDBACK = 0x2A1         #机械臂状态反馈ID
    ARM_END_POSE_FEEDBACK_1 = 0x2A2     #机械臂末端姿态反馈
    ARM_END_POSE_FEEDBACK_2 = 0x2A3
    ARM_END_POSE_FEEDBACK_3 = 0x2A4
    ARM_JOINT_FEEDBACK_12 = 0x2A5       #机械臂关节反馈
    ARM_JOINT_FEEDBACK_34 = 0x2A6
    ARM_JOINT_FEEDBACK_56 = 0x2A7
    ARM_GRIPPER_FEEDBACK = 0x2A8        #机械臂夹爪反馈
    #---------------------------------transmit------------------------------------#
    # 运动控制指令，可设置整体偏移为 0x160~0x169或 0x170~0x179，详见指令0x470
    ARM_MOTION_CTRL_1 = 0x150
    # ARM_STOP_CTRL = 0x150               #机械臂快速急停
    # ARM_TRACK_CTRL = 0x150              #机械臂轨迹指令
    # ARM_GRAG_TEACH_CTRL = 0x150         #机械臂拖动示教指令
    ARM_MOTION_CTRL_2 = 0x151
    # ARM_MODE_CTRL = 0x151               #机械臂控制模式
    # ARM_MOVE_MODE_CTRL = 0x151          #机械臂Mode模式
    # ARM_MOVE_SPD_RATE_CTRL = 0x151      #机械臂运动速度百分比
    ARM_MOTION_CTRL_CARTESIAN_1=0x152#机械臂运动控制直角坐标系指令1,X&Y
    ARM_MOTION_CTRL_CARTESIAN_2=0x153#机械臂运动控制直角坐标系指令1,Z&RX
    ARM_MOTION_CTRL_CARTESIAN_3=0x154#机械臂运动控制直角坐标系指令1,RY&RZ
    ARM_JOINT_CTRL_12=0x155             #机械臂臂部关节控制指令12,J1&J2
    ARM_JOINT_CTRL_34=0x156             #机械臂臂部关节控制指令12,J3&J4
    ARM_JOINT_CTRL_56=0x157             #机械臂臂部关节控制指令12,J5&J6
    ARM_CIRCULAR_PATTERN_COORD_NUM_UPDATE_CTRL=0x158#圆弧模式坐标序号更新指令数据
    ARM_GRIPPER_CTRL = 0x159            #夹爪控制指令
    # 机械臂参数配置与设定指令
    # 若指令名称带有反馈、应答; 决策控制单元->机械臂主控
    # 若指令名称带有查询、设置; 机械臂主控->决策控制单元
    ARM_MASTER_SLAVE_MODE_CONFIG = 0x470
    # ARM_MS_LINKAGE_CONFIG = 0x470          #随动主从模式设置指令-联动设置指令
    # ARM_MS_FEEDBACK_INSTRUCTION_OFFSET_CONFIG = 0x470#随动主从模式设置指令-反馈指令偏移值
    # ARM_MS_CTRL_INSTRUCTION_OFFSET_CONFIG = 0x470#随动主从模式设置指令-控制指令偏移
    # ARM_MS_LINKAGE_CTRL_OFFSET_CONFIG = 0x470#随动主从模式设置指令-联动模式控制目标地址偏移值
    # 设置为示教输入臂后，主动周期反馈报文 ID 增加偏移（偏移量可设置），模式切换为联动示教输入模式，不响应控制指令，且会主动发送关节模式控制指令；
    # 设置为运动输出臂后恢复为常规状态（退出联动示教输入模式，进入待机模式）；未收到此条指令的机械臂默认为此状态
    ARM_MOTOR_ENABLE_DISABLE_CONFIG = 0x471     #电机使能指令
    # ARM_MOTOR_DISABLE_CONFIG = 0x471    #电机失能指令
    # ARM_SEARCH_MOTOR_ANGLE_CONFIG = 0x472 #查询电机角度
    ARM_SEARCH_MOTOR_MAX_SPD_ACC_LIMIT = 0x472 #查询电机角度/最大速度/加速度限制
    # ARM_SEARCH_MOTOR_MAX_ACC_CONFIG = 0x472 #查询电机最大加速度限制
    ARM_FEEDBACK_CURRENT_MOTOR_ANGLE_LIMIT_MAX_SPD = 0x473 #反馈当前电机最大角度限制,最小角度限制,最大关节速度
    ARM_MOTOR_ANGLE_LIMIT_MAX_SPD_SET = 0x474      #电机角度限制/最大速度设置指令
    ARM_JOINT_CONFIG = 0x475            #关节设置指令
    ARM_INSTRUCTION_RESPONSE_CONFIG=0x476#设置指令应答
    ARM_PARAM_ENQUIRY_AND_CONFIG = 0x477#机械臂参数查询与设置指令
    ARM_FEEDBACK_CURRENT_END_VEL_ACC_PARAM = 0x478    #反馈当前末端速度/加速度参数
    ARM_END_VEL_ACC_PARAM_CONFIG = 0x479      #末端速度/加速度参数设置指令
    ARM_CRASH_PROTECTION_RATING_CONFIG=0x47A#碰撞防护等级设置指令
    ARM_CRASH_PROTECTION_RATING_FEEDBACK=0x47B#碰撞防护等级反馈指令
    ARM_FEEDBACK_CURRENT_MOTOR_MAX_ACC_LIMIT=0x47C#反馈当前电机最大加速度限制
    ARM_FEEDBACK_JOINT_VEL_ACC_1 = 0x481         #反馈当前关节的末端速度/加速度
    ARM_FEEDBACK_JOINT_VEL_ACC_2 = 0x482
    ARM_FEEDBACK_JOINT_VEL_ACC_3 = 0x483
    ARM_FEEDBACK_JOINT_VEL_ACC_4 = 0x484
    ARM_FEEDBACK_JOINT_VEL_ACC_5 = 0x485
    ARM_FEEDBACK_JOINT_VEL_ACC_6 = 0x486
    #灯光控制0x1节点ID，帧ID 0x121
    ARM_LIGHT_CTRL = 0x121              #灯光控制指令
    #驱动器信息高速反馈，节点ID 0x1~0x6
    ARM_INFO_HIGH_SPD_FEEDBACK_1 = 0x251
    ARM_INFO_HIGH_SPD_FEEDBACK_2 = 0x252
    ARM_INFO_HIGH_SPD_FEEDBACK_3 = 0x253
    ARM_INFO_HIGH_SPD_FEEDBACK_4 = 0x254
    ARM_INFO_HIGH_SPD_FEEDBACK_5 = 0x255
    ARM_INFO_HIGH_SPD_FEEDBACK_6 = 0x256
    #驱动器信息低速反馈，节点ID 0x1~0x6
    ARM_INFO_LOW_SPD_FEEDBACK_1 = 0x261
    ARM_INFO_LOW_SPD_FEEDBACK_2 = 0x262
    ARM_INFO_LOW_SPD_FEEDBACK_3 = 0x263
    ARM_INFO_LOW_SPD_FEEDBACK_4 = 0x264
    ARM_INFO_LOW_SPD_FEEDBACK_5 = 0x265
    ARM_INFO_LOW_SPD_FEEDBACK_6 = 0x266
    #CAN 升级总线静默模式设定指令
    ARM_CAN_UPDATE_SILENT_MODE_CONFIG=0x422
    # 固件读取指令
    ARM_FIRMWARE_READ = 0x4AF
    def __str__(self):
        return f"{self.name} (0x{self.value:X})"
    def __repr__(self):
        return f"{self.name}: 0x{self.value:X}"


#!/usr/bin/env python3
# -*-coding:utf8-*-
#机械臂协议V1版本，为方便后续修改协议升级，继承自base

from abc import ABC, abstractmethod
import time
from threading import Timer
from enum import Enum, auto
import can
from can.message import Message

from typing import (
    Optional,
)

# import sys,os
# sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '../../')))
from ..piper_protocol_base import C_PiperParserBase
from ...piper_msgs.msg_v1 import (
    ArmMsgType, 
    PiperMessage, 
    ArmMsgStatus, 
    ArmMsgJointFeedBack, 
    ArmMsgGripperFeedBack, 
    CanIDPiper,
    ArmMessageMapping
)

class C_PiperParserV1(C_PiperParserBase):
    '''
    Piper机械臂解析数据类V1版本
    '''
    '''
    Piper Robotic Arm Data Parsing Class V1 Version
    '''
    def __init__(self) -> None:
        pass

    def GetParserProtocolVersion(self):
        '''
        获取当前协议版本,当前为V1
        '''
        '''
        Get the current protocol version, currently V1.
        '''
        return self.ProtocolVersion.ARM_PROROCOL_V1

    def DecodeMessage(self, rx_can_frame: Optional[can.Message], msg:PiperMessage):
        '''解码消息,将can数据帧转为设定的类型

        Args:
            rx_can_frame (Optional[can.Message]): can 数据帧, 为输入
            msg (PiperMessage): 自定义中间层数据, 为输出

        Returns:
            bool:
                can消息的id如果存在, 反馈True

                can消息的id若不存在, 反馈False
        '''
        '''Decode the message, convert the CAN data frame to the specified type.

        Args:

            rx_can_frame (Optional[can.Message]): CAN data frame, input.
            msg (PiperMessage): Custom intermediate data, output.

        Returns:

            bool:
                If the CAN message ID exists, return True.
                If the CAN message ID does not exist, return False.
        '''
        ret:bool = True
        can_id:int = rx_can_frame.arbitration_id
        can_data:bytearray = rx_can_frame.data
        # 机械臂状态反馈,piper Status Feedback
        if(can_id == CanIDPiper.ARM_STATUS_FEEDBACK.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_status_msgs.ctrl_mode = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,0,1),False)
            msg.arm_status_msgs.arm_status = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,1,2),False)
            msg.arm_status_msgs.mode_feed = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,2,3),False)
            msg.arm_status_msgs.teach_status = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,3,4),False)
            msg.arm_status_msgs.motion_status = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,4,5),False)
            msg.arm_status_msgs.trajectory_num = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,5,6),False)
            msg.arm_status_msgs.err_code = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,6,8),False)
        # 机械臂末端位姿,piper End-Effector Pose
        elif(can_id == CanIDPiper.ARM_END_POSE_FEEDBACK_1.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_end_pose.X_axis = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.arm_end_pose.Y_axis = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_END_POSE_FEEDBACK_2.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_end_pose.Z_axis = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.arm_end_pose.RX_axis = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_END_POSE_FEEDBACK_3.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_end_pose.RY_axis = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.arm_end_pose.RZ_axis = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        # 关节角度反馈,Joint Angle Feedback
        elif(can_id == CanIDPiper.ARM_JOINT_FEEDBACK_12.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_joint_feedback.joint_1 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.arm_joint_feedback.joint_2 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_JOINT_FEEDBACK_34.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_joint_feedback.joint_3 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.arm_joint_feedback.joint_4 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_JOINT_FEEDBACK_56.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_joint_feedback.joint_5 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.arm_joint_feedback.joint_6 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        # 夹爪反馈,Gripper Feedback
        elif(can_id == CanIDPiper.ARM_GRIPPER_FEEDBACK.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.gripper_feedback.grippers_angle = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.gripper_feedback.grippers_effort = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,4,6))
            msg.gripper_feedback.status_code = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,6,7),False)
            # print(rx_can_frame)
        # 驱动器信息高速反馈,High-Speed Driver Information Feedback
        elif(can_id == CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_1.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_high_spd_feedback_1.can_id = can_id
            msg.arm_high_spd_feedback_1.motor_speed = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2))
            msg.arm_high_spd_feedback_1.current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4),False)
            msg.arm_high_spd_feedback_1.pos = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_2.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_high_spd_feedback_2.can_id = can_id
            msg.arm_high_spd_feedback_2.motor_speed = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2))
            msg.arm_high_spd_feedback_2.current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4),False)
            msg.arm_high_spd_feedback_2.pos = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_3.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_high_spd_feedback_3.can_id = can_id
            msg.arm_high_spd_feedback_3.motor_speed = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2))
            msg.arm_high_spd_feedback_3.current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4),False)
            msg.arm_high_spd_feedback_3.pos = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_4.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_high_spd_feedback_4.can_id = can_id
            msg.arm_high_spd_feedback_4.motor_speed = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2))
            msg.arm_high_spd_feedback_4.current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4),False)
            msg.arm_high_spd_feedback_4.pos = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_5.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_high_spd_feedback_5.can_id = can_id
            msg.arm_high_spd_feedback_5.motor_speed = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2))
            msg.arm_high_spd_feedback_5.current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4),False)
            msg.arm_high_spd_feedback_5.pos = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_6.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_high_spd_feedback_6.can_id = can_id
            msg.arm_high_spd_feedback_6.motor_speed = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2))
            msg.arm_high_spd_feedback_6.current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4),False)
            msg.arm_high_spd_feedback_6.pos = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        # 驱动器信息低速反馈,Low-Speed Driver Information Feedback
        elif(can_id == CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_1.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_low_spd_feedback_1.can_id = can_id
            msg.arm_low_spd_feedback_1.vol = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2),False)
            msg.arm_low_spd_feedback_1.foc_temp = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4))
            msg.arm_low_spd_feedback_1.motor_temp = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,4,5))
            msg.arm_low_spd_feedback_1.foc_status_code = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,5,6),False)
            msg.arm_low_spd_feedback_1.bus_current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,6,8),False)
        elif(can_id == CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_2.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_low_spd_feedback_2.can_id = can_id
            msg.arm_low_spd_feedback_2.vol = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2),False)
            msg.arm_low_spd_feedback_2.foc_temp = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4))
            msg.arm_low_spd_feedback_2.motor_temp = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,4,5))
            msg.arm_low_spd_feedback_2.foc_status_code = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,5,6),False)
            msg.arm_low_spd_feedback_2.bus_current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,6,8),False)
        elif(can_id == CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_3.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_low_spd_feedback_3.can_id = can_id
            msg.arm_low_spd_feedback_3.vol = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2),False)
            msg.arm_low_spd_feedback_3.foc_temp = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4))
            msg.arm_low_spd_feedback_3.motor_temp = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,4,5))
            msg.arm_low_spd_feedback_3.foc_status_code = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,5,6),False)
            msg.arm_low_spd_feedback_3.bus_current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,6,8),False)
        elif(can_id == CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_4.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_low_spd_feedback_4.can_id = can_id
            msg.arm_low_spd_feedback_4.vol = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2),False)
            msg.arm_low_spd_feedback_4.foc_temp = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4))
            msg.arm_low_spd_feedback_4.motor_temp = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,4,5))
            msg.arm_low_spd_feedback_4.foc_status_code = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,5,6),False)
            msg.arm_low_spd_feedback_4.bus_current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,6,8),False)
        elif(can_id == CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_5.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_low_spd_feedback_5.can_id = can_id
            msg.arm_low_spd_feedback_5.vol = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2),False)
            msg.arm_low_spd_feedback_5.foc_temp = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4))
            msg.arm_low_spd_feedback_5.motor_temp = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,4,5))
            msg.arm_low_spd_feedback_5.foc_status_code = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,5,6),False)
            msg.arm_low_spd_feedback_5.bus_current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,6,8),False)
        elif(can_id == CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_6.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_low_spd_feedback_6.can_id = can_id
            msg.arm_low_spd_feedback_6.vol = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2),False)
            msg.arm_low_spd_feedback_6.foc_temp = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4))
            msg.arm_low_spd_feedback_6.motor_temp = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,4,5))
            msg.arm_low_spd_feedback_6.foc_status_code = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,5,6),False)
            msg.arm_low_spd_feedback_6.bus_current = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,6,8),False)
        elif(can_id == CanIDPiper.ARM_FEEDBACK_CURRENT_MOTOR_ANGLE_LIMIT_MAX_SPD.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_feedback_current_motor_angle_limit_max_spd.motor_num = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,0,1),False)
            msg.arm_feedback_current_motor_angle_limit_max_spd.max_angle_limit = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,1,3))
            msg.arm_feedback_current_motor_angle_limit_max_spd.min_angle_limit = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,3,5))
            msg.arm_feedback_current_motor_angle_limit_max_spd.max_joint_spd = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,5,7),False)
        elif(can_id == CanIDPiper.ARM_FEEDBACK_CURRENT_END_VEL_ACC_PARAM.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_feedback_current_end_vel_acc_param.end_max_linear_vel = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,0,2),False)
            msg.arm_feedback_current_end_vel_acc_param.end_max_angular_vel = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,2,4),False)
            msg.arm_feedback_current_end_vel_acc_param.end_max_linear_acc = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,4,6),False)
            msg.arm_feedback_current_end_vel_acc_param.end_max_angular_acc = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,6,8),False)
        elif(can_id == CanIDPiper.ARM_CRASH_PROTECTION_RATING_FEEDBACK.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_crash_protection_rating_feedback.joint_1_protection_level = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,0,1),False)
            msg.arm_crash_protection_rating_feedback.joint_2_protection_level = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,1,2),False)
            msg.arm_crash_protection_rating_feedback.joint_3_protection_level = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,2,3),False)
            msg.arm_crash_protection_rating_feedback.joint_4_protection_level = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,3,4),False)
            msg.arm_crash_protection_rating_feedback.joint_5_protection_level = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,4,5),False)
            msg.arm_crash_protection_rating_feedback.joint_6_protection_level = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,5,6),False)
        elif(can_id == CanIDPiper.ARM_FEEDBACK_CURRENT_MOTOR_MAX_ACC_LIMIT.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_feedback_current_motor_max_acc_limit.joint_motor_num = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,0,1),False)
            msg.arm_feedback_current_motor_max_acc_limit.max_joint_acc = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,1,3),False)
        # 机械臂控制指令2,0x151
        elif(can_id == CanIDPiper.ARM_MOTION_CTRL_2.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_motion_ctrl_2.ctrl_mode = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,0,1),False)
            msg.arm_motion_ctrl_2.move_mode = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,1,2),False)
            msg.arm_motion_ctrl_2.move_spd_rate_ctrl = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,2,3),False)
            msg.arm_motion_ctrl_2.mit_mode = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,3,4),False)
            msg.arm_motion_ctrl_2.residence_time = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,4,5),False)
        # 读取主臂发送的目标joint数值
        elif(can_id == CanIDPiper.ARM_JOINT_CTRL_12.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_joint_ctrl.joint_1 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.arm_joint_ctrl.joint_2 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_JOINT_CTRL_34.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_joint_ctrl.joint_3 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.arm_joint_ctrl.joint_4 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        elif(can_id == CanIDPiper.ARM_JOINT_CTRL_56.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_joint_ctrl.joint_5 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.arm_joint_ctrl.joint_6 = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,4,8))
        # 夹爪
        elif(can_id == CanIDPiper.ARM_GRIPPER_CTRL.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.arm_gripper_ctrl.grippers_angle = self.ConvertToNegative_32bit(self.ConvertBytesToInt(can_data,0,4))
            msg.arm_gripper_ctrl.grippers_effort = self.ConvertToNegative_16bit(self.ConvertBytesToInt(can_data,4,6))
            msg.arm_gripper_ctrl.status_code = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,6,7),False)
            msg.arm_gripper_ctrl.set_zero = self.ConvertToNegative_8bit(self.ConvertBytesToInt(can_data,7,8),False)
        elif(can_id == CanIDPiper.ARM_FIRMWARE_READ.value):
            msg.type_ = ArmMessageMapping.get_mapping(can_id=can_id)
            msg.firmware_data = can_data
        else:
            ret = False
        return ret

    def EncodeMessage(self, msg:PiperMessage, tx_can_frame: Optional[can.Message]):
        '''将消息转为can数据帧

        Args:
            msg (PiperMessage): 自定义数据
            tx_can_frame (Optional[can.Message]): can要发送的数据

        Returns:
            bool:
                msg消息的type如果存在, 反馈True

                msg消息的type若不存在, 反馈False
        '''
        '''Convert the message to CAN data frame

        Args:
            msg (PiperMessage): Custom data
            tx_can_frame (Optional[can.Message]): CAN data to be sent

        Returns:
            bool:
                Returns True if the msg message type exists
                Returns False if the msg message type does not exist
        '''
        ret:bool = True
        msg_type_ = msg.type_
        tx_can_frame.arbitration_id = ArmMessageMapping.get_mapping(msg_type=msg_type_)
        if(msg_type_ == ArmMsgType.PiperMsgMotionCtrl_1):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_motion_ctrl_1.emergency_stop,False) + \
                                self.ConvertToList_8bit(msg.arm_motion_ctrl_1.track_ctrl,False) + \
                                self.ConvertToList_8bit(msg.arm_motion_ctrl_1.grag_teach_ctrl,False) + \
                                [0x00, 0x00, 0x00, 0x00, 0x00]
        elif(msg_type_ == ArmMsgType.PiperMsgMotionCtrl_2):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_motion_ctrl_2.ctrl_mode,False) + \
                                self.ConvertToList_8bit(msg.arm_motion_ctrl_2.move_mode,False) + \
                                self.ConvertToList_8bit(msg.arm_motion_ctrl_2.move_spd_rate_ctrl,False) + \
                                self.ConvertToList_8bit(msg.arm_motion_ctrl_2.mit_mode,False) + \
                                self.ConvertToList_8bit(msg.arm_motion_ctrl_2.residence_time,False) + \
                                [0x00, 0x00, 0x00]
        elif(msg_type_ == ArmMsgType.PiperMsgMotionCtrlCartesian_1):
            tx_can_frame.data = self.ConvertToList_32bit(msg.arm_motion_ctrl_cartesian.X_axis) + \
                                self.ConvertToList_32bit(msg.arm_motion_ctrl_cartesian.Y_axis)
        elif(msg_type_ == ArmMsgType.PiperMsgMotionCtrlCartesian_2):
            tx_can_frame.data = self.ConvertToList_32bit(msg.arm_motion_ctrl_cartesian.Z_axis) + \
                                self.ConvertToList_32bit(msg.arm_motion_ctrl_cartesian.RX_axis)
        elif(msg_type_ == ArmMsgType.PiperMsgMotionCtrlCartesian_3):
            tx_can_frame.data = self.ConvertToList_32bit(msg.arm_motion_ctrl_cartesian.RY_axis) + \
                                self.ConvertToList_32bit(msg.arm_motion_ctrl_cartesian.RZ_axis)
        elif(msg_type_ == ArmMsgType.PiperMsgJointCtrl_12):
            tx_can_frame.data = self.ConvertToList_32bit(msg.arm_joint_ctrl.joint_1) + \
                                self.ConvertToList_32bit(msg.arm_joint_ctrl.joint_2)
        elif(msg_type_ == ArmMsgType.PiperMsgJointCtrl_34):
            tx_can_frame.data = self.ConvertToList_32bit(msg.arm_joint_ctrl.joint_3) + \
                                self.ConvertToList_32bit(msg.arm_joint_ctrl.joint_4)
        elif(msg_type_ == ArmMsgType.PiperMsgJointCtrl_56):
            tx_can_frame.data = self.ConvertToList_32bit(msg.arm_joint_ctrl.joint_5) + \
                                self.ConvertToList_32bit(msg.arm_joint_ctrl.joint_6)
        elif(msg_type_ == ArmMsgType.PiperMsgCircularPatternCoordNumUpdateCtrl):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_circular_ctrl.instruction_num,False) + \
                                [0, 0, 0, 0, 0, 0, 0]
        elif(msg_type_ == ArmMsgType.PiperMsgGripperCtrl):
            tx_can_frame.data = self.ConvertToList_32bit(msg.arm_gripper_ctrl.grippers_angle) + \
                                self.ConvertToList_16bit(msg.arm_gripper_ctrl.grippers_effort,False) + \
                                self.ConvertToList_8bit(msg.arm_gripper_ctrl.status_code,False) + \
                                self.ConvertToList_8bit(msg.arm_gripper_ctrl.set_zero,False)
        elif(msg_type_ == ArmMsgType.PiperMsgMasterSlaveModeConfig):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_ms_config.linkage_config,False) + \
                                self.ConvertToList_8bit(msg.arm_ms_config.feedback_offset,False) + \
                                self.ConvertToList_8bit(msg.arm_ms_config.ctrl_offset,False) + \
                                self.ConvertToList_8bit(msg.arm_ms_config.linkage_offset,False) + \
                                [0, 0, 0, 0]
        elif(msg_type_ == ArmMsgType.PiperMsgMotorEnableDisableConfig):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_motor_enable.motor_num,False) + \
                                self.ConvertToList_8bit(msg.arm_motor_enable.enable_flag,False) + \
                                [0, 0, 0, 0, 0, 0]
        elif(msg_type_ == ArmMsgType.PiperMsgSearchMotorMaxAngleSpdAccLimit):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_search_motor_max_angle_spd_acc_limit.motor_num,False) + \
                                self.ConvertToList_8bit(msg.arm_search_motor_max_angle_spd_acc_limit.search_content,False) + \
                                [0, 0, 0, 0, 0, 0]
        elif(msg_type_ == ArmMsgType.PiperMsgMotorAngleLimitMaxSpdSet):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_motor_angle_limit_max_spd_set.motor_num,False) + \
                                self.ConvertToList_16bit(msg.arm_motor_angle_limit_max_spd_set.max_angle_limit) + \
                                self.ConvertToList_16bit(msg.arm_motor_angle_limit_max_spd_set.min_angle_limit) + \
                                self.ConvertToList_16bit(msg.arm_motor_angle_limit_max_spd_set.max_joint_spd,False) + \
                                [0]
        elif(msg_type_ == ArmMsgType.PiperMsgJointConfig):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_joint_config.joint_motor_num,False) + \
                                self.ConvertToList_8bit(msg.arm_joint_config.set_motor_current_pos_as_zero,False) + \
                                self.ConvertToList_8bit(msg.arm_joint_config.acc_param_config_is_effective_or_not,False) + \
                                self.ConvertToList_16bit(msg.arm_joint_config.max_joint_acc,False) + \
                                self.ConvertToList_8bit(msg.arm_joint_config.clear_joint_err,False) + \
                                [0, 0]
        elif(msg_type_ == ArmMsgType.PiperMsgInstructionResponseConfig):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_set_instruction_response.instruction_index,False) + \
                                self.ConvertToList_8bit(msg.arm_set_instruction_response.zero_config_success_flag,False) + \
                                [0, 0, 0, 0, 0, 0]
        elif(msg_type_ == ArmMsgType.PiperMsgParamEnquiryAndConfig):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_param_enquiry_and_config.param_enquiry,False) + \
                                self.ConvertToList_8bit(msg.arm_param_enquiry_and_config.param_setting,False) + \
                                self.ConvertToList_8bit(msg.arm_param_enquiry_and_config.data_feedback_0x48x,False) + \
                                self.ConvertToList_8bit(msg.arm_param_enquiry_and_config.end_load_param_setting_effective,False) + \
                                self.ConvertToList_8bit(msg.arm_param_enquiry_and_config.set_end_load,False) + \
                                [0, 0, 0]
        elif(msg_type_ == ArmMsgType.PiperMsgEndVelAccParamConfig):
            tx_can_frame.data = self.ConvertToList_16bit(msg.arm_end_vel_acc_config.end_max_linear_vel,False) + \
                                self.ConvertToList_16bit(msg.arm_end_vel_acc_config.end_max_angular_vel,False) + \
                                self.ConvertToList_16bit(msg.arm_end_vel_acc_config.end_max_linear_acc,False) + \
                                self.ConvertToList_16bit(msg.arm_end_vel_acc_config.end_max_angular_acc,False)
        elif(msg_type_ == ArmMsgType.PiperMsgCrashProtectionRatingConfig):
            tx_can_frame.data = self.ConvertToList_8bit(msg.arm_crash_protection_rating_config.joint_1_protection_level,False) + \
                                self.ConvertToList_8bit(msg.arm_crash_protection_rating_config.joint_2_protection_level,False) + \
                                self.ConvertToList_8bit(msg.arm_crash_protection_rating_config.joint_3_protection_level,False) + \
                                self.ConvertToList_8bit(msg.arm_crash_protection_rating_config.joint_4_protection_level,False) + \
                                self.ConvertToList_8bit(msg.arm_crash_protection_rating_config.joint_5_protection_level,False) + \
                                self.ConvertToList_8bit(msg.arm_crash_protection_rating_config.joint_6_protection_level,False) + \
                                [0, 0]
        else:
            ret = False
        return ret
            




#!/usr/bin/env python3
# -*-coding:utf8-*-

class ArmMsgCrashProtectionRatingConfig:
    '''
    msg_v2_transmit
    
    碰撞防护等级设置指令
    
    CAN ID:
        0x47A

    有效值 : 0~8

    等级 0 代表不检测碰撞； 6个关节可以独立设置

    Args:
        joint_1_protection_level: 关节1的碰撞等级设定
        joint_2_protection_level: 关节2的碰撞等级设定
        joint_3_protection_level: 关节3的碰撞等级设定
        joint_4_protection_level: 关节4的碰撞等级设定
        joint_5_protection_level: 关节5的碰撞等级设定
        joint_6_protection_level: 关节6的碰撞等级设定
    
    位描述:
    
        Byte 0: 1 号关节碰撞防护等级, uint8
        Byte 1: 2 号关节碰撞防护等级, uint8
        Byte 2: 3 号关节碰撞防护等级, uint8
        Byte 3: 4 号关节碰撞防护等级, uint8
        Byte 4: 5 号关节碰撞防护等级, uint8
        Byte 5: 6 号关节碰撞防护等级, uint8
        Byte 6: 保留
        Byte 7: 保留
    '''
    '''
    msg_v2_transmit
    
    End Effector Speed/Acceleration Parameter Setting Command

    CAN ID:
        0x47A

    Valid Values: 0~8
        Level 0 indicates no collision detection.
        Collision protection levels can be set independently for the six joints.

    Args:
        joint_1_protection_level: Collision protection level for Joint 1.
        joint_2_protection_level: Collision protection level for Joint 2.
        joint_3_protection_level: Collision protection level for Joint 3.
        joint_4_protection_level: Collision protection level for Joint 4.
        joint_5_protection_level: Collision protection level for Joint 5.
        joint_6_protection_level: Collision protection level for Joint 6.

    Bit Description:

        Byte 0: Collision protection level for Joint 1, uint8.
        Byte 1: Collision protection level for Joint 2, uint8.
        Byte 2: Collision protection level for Joint 3, uint8.
        Byte 3: Collision protection level for Joint 4, uint8.
        Byte 4: Collision protection level for Joint 5, uint8.
        Byte 5: Collision protection level for Joint 6, uint8.
        Byte 6: Reserved.
        Byte 7: Reserved.
    '''
    def __init__(self, 
                 joint_1_protection_level: int = 0xFF, 
                 joint_2_protection_level: int = 0xFF, 
                 joint_3_protection_level: int = 0xFF,
                 joint_4_protection_level: int = 0xFF,
                 joint_5_protection_level: int = 0xFF,
                 joint_6_protection_level: int = 0xFF
                 ):
        self.joint_1_protection_level = joint_1_protection_level
        self.joint_2_protection_level = joint_2_protection_level
        self.joint_3_protection_level = joint_3_protection_level
        self.joint_4_protection_level = joint_4_protection_level
        self.joint_5_protection_level = joint_5_protection_level
        self.joint_6_protection_level = joint_6_protection_level

    def __str__(self):
        return (f"ArmMsgCrashProtectionRatingConfig(\n"
                f"  joint_1_protection_level: {self.joint_1_protection_level},\n"
                f"  joint_2_protection_level: {self.joint_2_protection_level},\n"
                f"  joint_3_protection_level: {self.joint_3_protection_level},\n"
                f"  joint_4_protection_level: {self.joint_4_protection_level},\n"
                f"  joint_5_protection_level: {self.joint_5_protection_level},\n"
                f"  joint_6_protection_level: {self.joint_6_protection_level}\n"
                f")")

    def __repr__(self):
        return self.__str__()



#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# V2版本sdk
# 设定安装位置为水平正装
# 如需设定为侧装左/右
# MotionCtrl_2(0x01,0x01,0,0,0,0x02)
# MotionCtrl_2(0x01,0x01,0,0,0,0x03)

import time
from piper_sdk import *

if __name__ == "__main__":
    piper = C_PiperInterface_V2("can0")
    piper.ConnectPort()
    piper.MotionCtrl_2(0x01,0x01,0,0,0,0x01)  
    


#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# piper机械臂圆弧模式demo
# 注意机械臂工作空间内不要有障碍

from typing import (
    Optional,
)
import time
from piper_sdk import *

def enable_fun(piper:C_PiperInterface):
    '''
    使能机械臂并检测使能状态,尝试5s,如果使能超时则退出程序
    '''
    enable_flag = False
    # 设置超时时间（秒）
    timeout = 5
    # 记录进入循环前的时间
    start_time = time.time()
    elapsed_time_flag = False
    while not (enable_flag):
        elapsed_time = time.time() - start_time
        print("--------------------")
        enable_flag = piper.GetArmLowSpdInfoMsgs().motor_1.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_2.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_3.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_4.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_5.foc_status.driver_enable_status and \
            piper.GetArmLowSpdInfoMsgs().motor_6.foc_status.driver_enable_status
        print("使能状态:",enable_flag)
        piper.EnableArm(7)
        piper.GripperCtrl(0,1000,0x01, 0)
        print("--------------------")
        # 检查是否超过超时时间
        if elapsed_time > timeout:
            print("超时....")
            elapsed_time_flag = True
            enable_flag = True
            break
        time.sleep(1)
        pass
    if(elapsed_time_flag):
        print("程序自动使能超时,退出程序")
        exit(0)

if __name__ == "__main__":
    piper = C_PiperInterface("can0")
    piper.ConnectPort()
    piper.EnableArm(7)
    enable_fun(piper=piper)
    # piper.DisableArm(7)
    piper.GripperCtrl(0,1000,0x01, 0)
    # X:135.481
    piper.EndPoseCtrl(135481,9349,161129,178756,6035,-178440)
    piper.MoveCAxisUpdateCtrl(0x01)
    time.sleep(0.001)
    piper.EndPoseCtrl(222158,128758,142126,175152,-1259,-157235)
    piper.MoveCAxisUpdateCtrl(0x02)
    time.sleep(0.001)
    piper.EndPoseCtrl(359079,3221,153470,179038,1105,179035)
    piper.MoveCAxisUpdateCtrl(0x03)
    time.sleep(0.001)
    piper.MotionCtrl_2(0x01, 0x03, 30, 0x00)
    pass


#!/usr/bin/env python3
# -*-coding:utf8-*-

from typing import (
    Optional,
)
from .arm_msg_type import ArmMsgType
from .can_id import CanIDPiper

class ArmMessageMapping:
    '''
    msg_v1
    
    机械臂消息类型和CAN ID的映射
    '''
    '''
    msg_v1
    
    Mapping of Robotic Arm Message Types and CAN IDs
    '''
    # 初始化映射字典
    id_to_type_mapping = {
        # feedback
        CanIDPiper.ARM_STATUS_FEEDBACK.value: ArmMsgType.PiperMsgStatusFeedback,
        CanIDPiper.ARM_END_POSE_FEEDBACK_1.value: ArmMsgType.PiperMsgEndPoseFeedback_1,
        CanIDPiper.ARM_END_POSE_FEEDBACK_2.value: ArmMsgType.PiperMsgEndPoseFeedback_2,
        CanIDPiper.ARM_END_POSE_FEEDBACK_3.value: ArmMsgType.PiperMsgEndPoseFeedback_3,
        CanIDPiper.ARM_JOINT_FEEDBACK_12.value: ArmMsgType.PiperMsgJointFeedBack_12,
        CanIDPiper.ARM_JOINT_FEEDBACK_34.value: ArmMsgType.PiperMsgJointFeedBack_34,
        CanIDPiper.ARM_JOINT_FEEDBACK_56.value: ArmMsgType.PiperMsgJointFeedBack_56,
        CanIDPiper.ARM_GRIPPER_FEEDBACK.value: ArmMsgType.PiperMsgGripperFeedBack,
        CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_1.value: ArmMsgType.PiperMsgHighSpdFeed_1,
        CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_2.value: ArmMsgType.PiperMsgHighSpdFeed_2,
        CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_3.value: ArmMsgType.PiperMsgHighSpdFeed_3,
        CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_4.value: ArmMsgType.PiperMsgHighSpdFeed_4,
        CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_5.value: ArmMsgType.PiperMsgHighSpdFeed_5,
        CanIDPiper.ARM_INFO_HIGH_SPD_FEEDBACK_6.value: ArmMsgType.PiperMsgHighSpdFeed_6,
        CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_1.value: ArmMsgType.PiperMsgLowSpdFeed_1,
        CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_2.value: ArmMsgType.PiperMsgLowSpdFeed_2,
        CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_3.value: ArmMsgType.PiperMsgLowSpdFeed_3,
        CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_4.value: ArmMsgType.PiperMsgLowSpdFeed_4,
        CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_5.value: ArmMsgType.PiperMsgLowSpdFeed_5,
        CanIDPiper.ARM_INFO_LOW_SPD_FEEDBACK_6.value: ArmMsgType.PiperMsgLowSpdFeed_6,
        # transmit
        CanIDPiper.ARM_MOTION_CTRL_1.value: ArmMsgType.PiperMsgMotionCtrl_1,
        # CanIDPiper.ARM_STOP_CTRL.value: ArmMsgType.PiperMsgStopCtrl,
        # CanIDPiper.ARM_TRACK_CTRL.value: ArmMsgType.PiperMsgTrackCtrl,
        # CanIDPiper.ARM_GRAG_TEACH_CTRL.value: ArmMsgType.PiperMsgGragTeachCtrl,
        CanIDPiper.ARM_MOTION_CTRL_2.value: ArmMsgType.PiperMsgMotionCtrl_2,
        # CanIDPiper.ARM_MODE_CTRL.value: ArmMsgType.PiperMsgModeCtrl,
        # CanIDPiper.ARM_MOVE_MODE_CTRL.value: ArmMsgType.PiperMsgMoveModeCtrl,
        # CanIDPiper.ARM_MOVE_SPD_RATE_CTRL.value: ArmMsgType.PiperMsgMoveSpdRateCtrl,
        CanIDPiper.ARM_MOTION_CTRL_CARTESIAN_1.value: ArmMsgType.PiperMsgMotionCtrlCartesian_1,
        CanIDPiper.ARM_MOTION_CTRL_CARTESIAN_2.value: ArmMsgType.PiperMsgMotionCtrlCartesian_2,
        CanIDPiper.ARM_MOTION_CTRL_CARTESIAN_3.value: ArmMsgType.PiperMsgMotionCtrlCartesian_3,
        CanIDPiper.ARM_JOINT_CTRL_12.value: ArmMsgType.PiperMsgJointCtrl_12,
        CanIDPiper.ARM_JOINT_CTRL_34.value: ArmMsgType.PiperMsgJointCtrl_34,
        CanIDPiper.ARM_JOINT_CTRL_56.value: ArmMsgType.PiperMsgJointCtrl_56,
        CanIDPiper.ARM_CIRCULAR_PATTERN_COORD_NUM_UPDATE_CTRL.value: ArmMsgType.PiperMsgCircularPatternCoordNumUpdateCtrl,
        CanIDPiper.ARM_GRIPPER_CTRL.value: ArmMsgType.PiperMsgGripperCtrl,
        CanIDPiper.ARM_MASTER_SLAVE_MODE_CONFIG.value: ArmMsgType.PiperMsgMasterSlaveModeConfig,
        # CanIDPiper.ARM_MS_LINKAGE_CONFIG.value: ArmMsgType.PiperMsgMSLinkageConfig,
        # CanIDPiper.ARM_MS_FEEDBACK_INSTRUCTION_OFFSET_CONFIG.value: ArmMsgType.PiperMsgMSFeedbackInstructionOffsetConfig,
        # CanIDPiper.ARM_MS_CTRL_INSTRUCTION_OFFSET_CONFIG.value: ArmMsgType.PiperMsgMSCtrlInstructionOffsetConfig,
        # CanIDPiper.ARM_MS_LINKAGE_CTRL_OFFSET_CONFIG.value: ArmMsgType.PiperMsgMSLinkageCtrlOffsetConfig,
        CanIDPiper.ARM_MOTOR_ENABLE_DISABLE_CONFIG.value: ArmMsgType.PiperMsgMotorEnableDisableConfig,
        # CanIDPiper.ARM_MOTOR_DISABLE_CONFIG.value: ArmMsgType.PiperMsgMotorDisableConfig,
        # CanIDPiper.ARM_SEARCH_MOTOR_ANGLE_CONFIG.value: ArmMsgType.PiperMsgSearchMotorAngleConfig,
        CanIDPiper.ARM_SEARCH_MOTOR_MAX_SPD_ACC_LIMIT.value: ArmMsgType.PiperMsgSearchMotorMaxAngleSpdAccLimit,
        # CanIDPiper.ARM_SEARCH_MOTOR_MAX_ACC_CONFIG.value: ArmMsgType.PiperMsgSearchMotorMaxAccConfig,
        CanIDPiper.ARM_FEEDBACK_CURRENT_MOTOR_ANGLE_LIMIT_MAX_SPD.value: ArmMsgType.PiperMsgFeedbackCurrentMotorAngleLimitMaxSpd,
        CanIDPiper.ARM_MOTOR_ANGLE_LIMIT_MAX_SPD_SET.value: ArmMsgType.PiperMsgMotorAngleLimitMaxSpdSet,
        CanIDPiper.ARM_JOINT_CONFIG.value: ArmMsgType.PiperMsgJointConfig,
        CanIDPiper.ARM_INSTRUCTION_RESPONSE_CONFIG.value: ArmMsgType.PiperMsgInstructionResponseConfig,
        CanIDPiper.ARM_PARAM_ENQUIRY_AND_CONFIG.value: ArmMsgType.PiperMsgParamEnquiryAndConfig,
        CanIDPiper.ARM_FEEDBACK_CURRENT_END_VEL_ACC_PARAM.value: ArmMsgType.PiperMsgFeedbackCurrentEndVelAccParam,
        CanIDPiper.ARM_END_VEL_ACC_PARAM_CONFIG.value: ArmMsgType.PiperMsgEndVelAccParamConfig,
        CanIDPiper.ARM_CRASH_PROTECTION_RATING_CONFIG.value: ArmMsgType.PiperMsgCrashProtectionRatingConfig,
        CanIDPiper.ARM_CRASH_PROTECTION_RATING_FEEDBACK.value: ArmMsgType.PiperMsgCrashProtectionRatingFeedback,
        CanIDPiper.ARM_FEEDBACK_CURRENT_MOTOR_MAX_ACC_LIMIT.value: ArmMsgType.PiperMsgFeedbackCurrentMotorMaxAccLimit,
        CanIDPiper.ARM_FEEDBACK_JOINT_VEL_ACC_1.value: ArmMsgType.PiperMsgFeedbackJointVelAcc_1,
        CanIDPiper.ARM_FEEDBACK_JOINT_VEL_ACC_2.value: ArmMsgType.PiperMsgFeedbackJointVelAcc_2,
        CanIDPiper.ARM_FEEDBACK_JOINT_VEL_ACC_3.value: ArmMsgType.PiperMsgFeedbackJointVelAcc_3,
        CanIDPiper.ARM_FEEDBACK_JOINT_VEL_ACC_4.value: ArmMsgType.PiperMsgFeedbackJointVelAcc_4,
        CanIDPiper.ARM_FEEDBACK_JOINT_VEL_ACC_5.value: ArmMsgType.PiperMsgFeedbackJointVelAcc_5,
        CanIDPiper.ARM_FEEDBACK_JOINT_VEL_ACC_6.value: ArmMsgType.PiperMsgFeedbackJointVelAcc_6,
        CanIDPiper.ARM_LIGHT_CTRL.value: ArmMsgType.PiperMsgLightCtrl,
        CanIDPiper.ARM_CAN_UPDATE_SILENT_MODE_CONFIG.value: ArmMsgType.PiperMsgCanUpdateSilentModeConfig,
        CanIDPiper.ARM_FIRMWARE_READ.value: ArmMsgType.PiperMsgFirmwareRead,
    }

    type_to_id_mapping = {v: k for k, v in id_to_type_mapping.items()}

    @staticmethod
    def get_mapping(can_id: Optional[int] = None, msg_type: Optional[ArmMsgType] = None):
        '''
        根据输入的参数返回对应的映射值，输入 id 返回类型，输入类型返回 id
        
        Args:
            can_id: CAN ID
            msg_type: 机械臂消息类型
        
        return: 对应的类型或 id
        '''
        if can_id is not None and msg_type is not None:
            raise ValueError("只能输入 CAN ID 或消息类型中的一个")

        if can_id is not None:
            if can_id in ArmMessageMapping.id_to_type_mapping:
                return ArmMessageMapping.id_to_type_mapping[can_id]
            else:
                raise ValueError(f"CAN ID {can_id} 不在映射中")

        if msg_type is not None:
            if msg_type in ArmMessageMapping.type_to_id_mapping:
                return ArmMessageMapping.type_to_id_mapping[msg_type]
            else:
                raise ValueError(f"消息类型 {msg_type} 不在映射中")

        raise ValueError("必须输入 CAN ID 或消息类型中的一个")

# 测试代码
# if __name__ == "__main__":
#     # 根据 ID 查找类型
#     print(ArmMessageMapping.get_mapping(can_id=0x2A2))  # 输出: PiperMsgEndPoseFeedback_1 (0x2)

#     # 根据类型查找 ID
#     print(ArmMessageMapping.get_mapping(msg_type=ArmMsgType.PiperMsgJointFeedBack_56))  # 输出: 0x2A7



#!/usr/bin/env python3
# -*-coding:utf8-*-
# 注意demo无法直接运行，需要pip安装sdk后才能运行
# 设置机械臂为mit控制模式，这个模式下，机械臂相应最快

from typing import (
    Optional,
)
import time
from piper_sdk import *

# 测试代码
if __name__ == "__main__":
    piper = C_PiperInterface()
    piper.ConnectPort()
    piper.MotionCtrl_2(0, 0, 0, 0xAD)# 0xFC



#!/usr/bin/env python3
# -*-coding:utf8-*-

#机械臂控制base

import can
from can.message import Message
from typing import (
    Optional,
)
from typing import Type

from ..interface.piper_interface import C_PiperInterface
from ..hardware_port.can_encapsulation import C_STD_CAN

class C_PiperBase(C_PiperInterface):
    def __init__(self, can_name: str = "can0") -> None:
        super().__init__(can_name)
    
    def Connect(self, can_name:str):
        return self.ConnectPort(can_name)
